@@ -18,6 +18,7 @@
 
 #include "gva-categories.h"
 
+#include <string.h>
 #include "gva-error.h"
 
 static GKeyFile *keyfile = NULL;
@@ -37,17 +38,35 @@ gboolean
 gva_categories_init (GError **error)
 {
         gboolean success = FALSE;
+        gchar *contents, *cp;
+        gsize length;
 
         keyfile = g_key_file_new ();
 
 #ifdef CATEGORY_FILE
-        success = g_key_file_load_from_file (
-                keyfile, CATEGORY_FILE, G_KEY_FILE_NONE, error);
+        if (!g_file_get_contents (CATEGORY_FILE, &contents, &length, error))
+                return FALSE;
+
+        /* Convert Windows INI-style comments (lines beginning with ';')
+         * to a form recognized by the GKeyFile parser. */
+        cp = contents;
+        while (cp != NULL && *cp != '\0') {
+                if (*cp == ';')
+                        *cp = '#';
+                /* Find the next line. */
+                cp = strchr (cp, '\n');
+                if (cp != NULL)
+                        cp++;
+        }
+
+        success = g_key_file_load_from_data (
+                keyfile, contents, length, G_KEY_FILE_NONE, error);
+
+        g_free (contents);
 #else
-        g_set_error (
-                error, GVA_ERROR, GVA_ERROR_CONFIG,
-                _("This program is not configured to show "
-                  "category or version information."));
+        g_message (
+                _("This program is not configured "
+                  "to show category information."));
 #endif
 
         return success;
