@@ -1,4 +1,4 @@
-/* Copyright 2007 Matthew Barnes
+/* Copyright 2007, 2008 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -19,6 +19,8 @@
 /**
  * SECTION: gva-tree-view
  * @short_description: Main Tree View
+ *
+ * These functions manipulate the tree view widget in the main window.
  **/
 
 #ifndef GVA_TREE_VIEW_H
@@ -41,8 +43,6 @@ gint           gva_tree_view_get_selected_view       (void);
 void           gva_tree_view_set_selected_view       (gint view);
 const gchar *  gva_tree_view_get_last_selected_game  (void);
 void           gva_tree_view_set_last_selected_game  (const gchar *game);
-gint           gva_tree_view_get_last_selected_view  (void);
-void           gva_tree_view_set_last_selected_view  (gint view);
 void           gva_tree_view_get_last_sort_column_id (GvaGameStoreColumn *column_id,
                                                       GtkSortType *order);
 void           gva_tree_view_set_last_sort_column_id (GvaGameStoreColumn column_id,
